MODULE trdicp
   !!======================================================================
   !!                       ***  MODULE  trdicp  ***
   !! Ocean diagnostics:  ocean tracers and dynamic trends
   !!=====================================================================
   !! History :       !  91-12 (G. Madec)
   !!                 !  92-06 (M. Imbard) add time step frequency
   !!                 !  96-01 (G. Madec)  terrain following coordinates
   !!            8.5  !  02-06 (G. Madec)  F90: Free form and module
   !!            9.0  !  04-08 (C. Talandier) New trends organization
   !!----------------------------------------------------------------------
#if  defined key_trdtra   ||   defined key_trddyn   ||   defined key_esopa
   !!----------------------------------------------------------------------
   !!   'key_trdtra'  or                  active tracers trends diagnostics
   !!   'key_trddyn'                            momentum trends diagnostics
   !!----------------------------------------------------------------------
   !!----------------------------------------------------------------------
   !!   trd_icp          : compute the basin averaged properties for tra/dyn 
   !!   trd_dwr          : print dynmaic trends in ocean.output file
   !!   trd_twr          : print tracers trends in ocean.output file
   !!   trd_icp_init     : initialization step
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and tracers variables
   USE dom_oce         ! ocean space and time domain variables
   USE trdmod_oce      ! ocean variables trends
   USE ldftra_oce      ! ocean active tracers: lateral physics
   USE ldfdyn_oce      ! ocean dynamics: lateral physics
   USE zdf_oce         ! ocean vertical physics
   USE in_out_manager  ! I/O manager
   USE lib_mpp         ! distibuted memory computing library
   USE eosbn2          ! equation of state
   USE phycst          ! physical constants

   IMPLICIT NONE
   PRIVATE

   INTERFACE trd_icp
      MODULE PROCEDURE trd_2d, trd_3d, trd_u2d, trd_u3d
   END INTERFACE

   PUBLIC   trd_icp       ! called by trdmod.F90
   PUBLIC   trd_dwr       ! called by step.F90
   PUBLIC   trd_twr       ! called by step.F90
   PUBLIC   trd_icp_init  ! called by opa.F90

   !! Variables used for diagnostics
   REAL(wp) ::   tvolt        !: volume of the whole ocean computed at t-points
   REAL(wp) ::   tvolu        !: volume of the whole ocean computed at u-points
   REAL(wp) ::   tvolv        !: volume of the whole ocean computed at v-points

   !! Active Tracer trend diagnostics variables
   REAL(wp), DIMENSION(jpt_trd,2) ::   tsmo         !: tracers trends average 
   REAL(wp), DIMENSION(jpt_trd,2) ::   ts2          !: tracers square trends average 

   !! Momentum trends diagnostics variables
   REAL(wp), DIMENSION(jptot_dyn) ::   umo, vmo         !: momentum trends average 
   REAL(wp), DIMENSION(jptot_dyn) ::   hke              !: momentum square trends average 
   REAL(wp) ::   rpktrd   !: potential to kinetic energy conversion
   REAL(wp) ::   peke     !: conversion potential energy - kinetic energy trend

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2005) 
   !! $Header$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trd_2d( ptrd, ktra, ktrd , ctype, clpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_2d  ***
      !! 
      !! ** Purpose : verify the basin averaged properties of tracers and/or
      !!              momentum equations at every time step frequency ntrd.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout)           ::   ptrd      ! tracer or u trend
      INTEGER                     , INTENT(in   )           ::   ktra      ! tracer index
      INTEGER                     , INTENT(in   )           ::   ktrd      ! tracer trend index
      CHARACTER(len=3)            , INTENT(in   )           ::   ctype     ! tracer type (='TRA' or 'TRC')
      CHARACTER(len=3)            , INTENT(in   ), OPTIONAL ::   clpas     ! number of passage
      !!
      CHARACTER(len=3) ::   cpas   ! number of passage
      REAL(wp)         ::   zsum   ! temporary scalars
      !!----------------------------------------------------------------------

      cpas = 'fst'                             ! Control of optional arguments
      IF( PRESENT(clpas) )  cpas = clpas

      ! 1. Mask volumic trends
      ptrd(:,:) = e1t(:,:) * e2t(:,:) * fse3t(:,:,1) * ptrd(:,:) * tmask_i(:,:)
      
      ! 2. Basin averaged tracer trends
      SELECT CASE( ctype )
      CASE( 'TRA' )              ! Tracers
         zsum = SUM( ptrd(:,:) )
         IF( cpas == 'fst' ) THEN   ;   tsmo(ktrd,ktra) =                   zsum
         ELSE                       ;   tsmo(ktrd,ktra) = tsmo(ktrd,ktra) + zsum
         ENDIF
      CASE( 'TRC' )              ! Passive tracers
         !      ....  to be done
      END SELECT
      
      ! 3. Basin averaged tracer square trends (i.e. trd * now tracer field)
      SELECT CASE( ctype )
      CASE( 'TRA' )              ! Tracers
         IF( ktra == jp_tem )   zsum = SUM(  ptrd(:,:) * tn(:,:,1) )
         IF( ktra == jp_sal )   zsum = SUM(  ptrd(:,:) * sn(:,:,1) )
         IF( cpas == 'fst' ) THEN   ;   ts2(ktrd,ktra) =                  zsum
         ELSE                       ;   ts2(ktrd,ktra) = ts2(ktrd,ktra) + zsum
         ENDIF
      CASE( 'TRC' )              ! Passive tracers
         !      ....  to be done
      END SELECT
      !
   END SUBROUTINE trd_2d


   SUBROUTINE trd_3d( ptrd, ktra, ktrd , ctype, clpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_3d  ***
      !! 
      !! ** Purpose : verify the basin averaged properties of tracers and/or
      !!              momentum equations at every time step frequency ntrd.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(inout)           ::   ptrd      ! tracer or u trend
      INTEGER                         , INTENT(in   )           ::   ktra      ! tracer index
      INTEGER                         , INTENT(in   )           ::   ktrd      ! tracer trend index
      CHARACTER(len=3)                , INTENT(in   )           ::   ctype     ! tracer type (='TRA' or 'TRC')
      CHARACTER(len=3)                , INTENT(in   ), OPTIONAL ::   clpas     ! number of passage
      !!
      INTEGER          ::   jk     ! dummy loop indices
      CHARACTER(len=3) ::   cpas   ! number of passage
      REAL(wp)         ::   zsum   ! temporary scalars
      REAL(wp), DIMENSION(jpi,jpj) ::   zsurf          ! 2D workspace array
      !!----------------------------------------------------------------------

      cpas = 'fst'                             ! Control of optional arguments
      IF( PRESENT(clpas) )  cpas = clpas

      ! 1. Mask volumic trends
      zsurf(:,:) = e1t(:,:) * e2t(:,:) * tmask_i(:,:)
      DO jk = 1, jpk
         ptrd(:,:,jk) = zsurf(:,:) * fse3t(:,:,jk) * ptrd(:,:,jk) * tmask(:,:,jk)
      END DO

      ! 2. Basin averaged tracer trends
      SELECT CASE( ctype )
      CASE( 'TRA' )              ! Tracers
         zsum = SUM( ptrd(:,:,:) )
         IF( cpas == 'fst' ) THEN   ;   tsmo(ktrd,ktra) =                   zsum
         ELSE                       ;   tsmo(ktrd,ktra) = tsmo(ktrd,ktra) + zsum
         ENDIF
      CASE( 'TRC' )              ! Passive tracers
         !      ....  to be done
      END SELECT

      ! 3. Basin averaged tracer square trends (i.e. trd * now tracer field)
      SELECT CASE( ctype )
      CASE( 'TRA' )              ! Tracers
         IF( ktra == jp_tem )   zsum = SUM(  ptrd(:,:,:) * tn(:,:,:) )
         IF( ktra == jp_sal )   zsum = SUM(  ptrd(:,:,:) * sn(:,:,:) )
         IF( cpas == 'fst' ) THEN   ;   ts2(ktrd,ktra) =                  zsum
         ELSE                       ;   ts2(ktrd,ktra) = ts2(ktrd,ktra) + zsum
         ENDIF
      CASE( 'TRC' )              ! Passive tracers
         !      ....  to be done
      END SELECT
      !
   END SUBROUTINE trd_3d


   SUBROUTINE trd_u2d( ptrdu, ptrdv, ktrd , ctype, clpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_2d  ***
      !! 
      !! ** Purpose : verify the basin averaged properties of tracers and/or
      !!              momentum equations at every time step frequency ntrd.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj), INTENT(inout)           ::   ptrdu, ptrdv   ! U and V momentum trends 
      INTEGER                     , INTENT(in   )           ::   ktrd           ! tracer trend index
      CHARACTER(len=3)            , INTENT(in   )           ::   ctype          ! momentum ('DYN')
      CHARACTER(len=3)            , INTENT(in   ), OPTIONAL ::   clpas          ! number of passage
      !!
      INTEGER          ::   ji, jj        ! loop indices
      CHARACTER(len=3) ::   cpas          ! number of passage
      REAL(wp)         ::   zmsku, zbtu   ! temporary scalars
      REAL(wp)         ::   zmskv, zbtv   !    "         "
      !!----------------------------------------------------------------------

      cpas = 'fst'                           ! Control of optional arguments
      IF( PRESENT(clpas) )  cpas = clpas

      SELECT CASE( ctype )
      !
      CASE( 'DYN' )              ! Momentum

      ! 1. Mask trends
      ! --------------
      DO jj = 1, jpjm1
         DO ji = 1, jpim1
            zmsku = tmask_i(ji+1,jj  ) * tmask_i(ji,jj) * umask(ji,jj,1)
            zmskv = tmask_i(ji  ,jj+1) * tmask_i(ji,jj) * vmask(ji,jj,1)
            ptrdu(ji,jj) = ptrdu(ji,jj) * zmsku
            ptrdv(ji,jj) = ptrdv(ji,jj) * zmskv
         END DO
      END DO
      ptrdu(jpi, : ) = 0.e0      ;      ptrdv(jpi, : ) = 0.e0
      ptrdu( : ,jpj) = 0.e0      ;      ptrdv( : ,jpj) = 0.e0
      
      ! 2. Basin averaged tracer/momentum trends
      ! ----------------------------------------
      umo(ktrd) = 0.e0
      vmo(ktrd) = 0.e0
      !
      SELECT CASE( ktrd )
      !
      CASE( jpdyn_trd_swf )         ! surface forcing
         DO jj = 1, jpj
            DO ji = 1, jpi
               umo(ktrd) = umo(ktrd) + ptrdu(ji,jj) * e1u(ji,jj) * e2u(ji,jj) * fse3u(ji,jj,1)
               vmo(ktrd) = vmo(ktrd) + ptrdv(ji,jj) * e1v(ji,jj) * e2v(ji,jj) * fse3v(ji,jj,1)
            END DO
         END DO
            !
         CASE( jpdyn_trd_bfr )         ! bottom friction fluxes
            DO jj = 1, jpj
               DO ji = 1, jpi
                  umo(ktrd) = umo(ktrd) + ptrdu(ji,jj)
                  vmo(ktrd) = vmo(ktrd) + ptrdv(ji,jj)
               END DO
            END DO
            !
      END SELECT
      !   
      ! 3. Basin averaged tracer/momentum square trends
      ! ----------------------------------------------
      ! c a u t i o n: field now
      
         hke(ktrd) = 0.e0
         DO jj = 1, jpj
            DO ji = 1, jpi
               zbtu = e1u(ji,jj) * e2u(ji,jj) * fse3u(ji,jj,1)
               zbtv = e1v(ji,jj) * e2v(ji,jj) * fse3v(ji,jj,1)
               hke(ktrd) = hke(ktrd) + un(ji,jj,1) * ptrdu(ji,jj) * zbtu   &
               &                     + vn(ji,jj,1) * ptrdv(ji,jj) * zbtv
            END DO
         END DO
         !
      END SELECT
      !
   END SUBROUTINE trd_u2d


   SUBROUTINE trd_u3d( ptrdu, ptrdv, ktrd, ctype, clpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_3d  ***
      !! 
      !! ** Purpose : verify the basin averaged properties of tracers and/or
      !!              momentum equations at every time step frequency ntrd.
      !!----------------------------------------------------------------------
      REAL(wp), DIMENSION(jpi,jpj,jpk), INTENT(inout)           ::   ptrdu, ptrdv   ! U and V momentum trends
      INTEGER,                          INTENT(in   )           ::   ktrd           ! momentum trend index
      CHARACTER(len=3),                 INTENT(in   )           ::   ctype          ! momentum (='DYN')
      CHARACTER(len=3),                 INTENT(in   ), OPTIONAL ::   clpas          ! number of passage
      !!
      INTEGER ::   ji, jj, jk
      CHARACTER(len=3) ::   cpas               ! number of passage
      REAL(wp) ::   zbtu, zbtv, zmsku, zmskv   ! temporary scalars
      !!----------------------------------------------------------------------

      cpas = 'fst'                            ! Control of optional arguments
      IF( PRESENT(clpas) )  cpas = clpas

      SELECT CASE( ctype )
      !
      CASE( 'DYN' )              ! Momentum        

         ! 1. Mask the trends
         ! ------------------
         DO jk = 1, jpk
            DO jj = 1, jpjm1
               DO ji = 1, jpim1
                  zmsku = tmask_i(ji+1,jj  ) * tmask_i(ji,jj) * umask(ji,jj,jk)
                  zmskv = tmask_i(ji  ,jj+1) * tmask_i(ji,jj) * vmask(ji,jj,jk)
                  ptrdu(ji,jj,jk) = ptrdu(ji,jj,jk) * zmsku
                  ptrdv(ji,jj,jk) = ptrdv(ji,jj,jk) * zmskv
               END DO
            END DO
         END DO
         ptrdu(jpi, : ,:) = 0.e0      ;      ptrdv(jpi, : ,:) = 0.e0
         ptrdu( : ,jpj,:) = 0.e0      ;      ptrdv( : ,jpj,:) = 0.e0
         !

      ! 2. Basin averaged tracer/momentum trends
      ! ----------------------------------------
      
         umo(ktrd) = 0.e0
         vmo(ktrd) = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zbtu = e1u(ji,jj) * e2u(ji,jj) * fse3u(ji,jj,jk)
                  zbtv = e1v(ji,jj) * e2v(ji,jj) * fse3v(ji,jj,jk)
                  umo(ktrd) = umo(ktrd) + ptrdu(ji,jj,jk) * zbtu
                  vmo(ktrd) = vmo(ktrd) + ptrdv(ji,jj,jk) * zbtv
               END DO
            END DO
         END DO
         !

      ! 3. Basin averaged tracer/momentum square trends
      ! -----------------------------------------------
      ! c a u t i o n: field now
      
         hke(ktrd) = 0.e0
         DO jk = 1, jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  zbtu = e1u(ji,jj) * e2u(ji,jj) * fse3u(ji,jj,jk)
                  zbtv = e1v(ji,jj) * e2v(ji,jj) * fse3v(ji,jj,jk)
                  hke(ktrd) = hke(ktrd) + un(ji,jj,jk) * ptrdu(ji,jj,jk) * zbtu   &
                  &                     + vn(ji,jj,jk) * ptrdv(ji,jj,jk) * zbtv
               END DO
            END DO
         END DO
         !
      END SELECT
      !
   END SUBROUTINE trd_u3d


   SUBROUTINE trd_icp_init
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_icp_init  ***
      !! 
      !! ** Purpose :   Read the namtrd namelist
      !!----------------------------------------------------------------------
      INTEGER  ::   jk
#if  defined key_trddyn
      INTEGER  ::   ji, jj
      REAL(wp) ::   zmsku, zmskv
#endif
      REAL(wp), DIMENSION(jpi,jpj) ::   zsurf          ! 2D workspace array
      !!----------------------------------------------------------------------

      IF(lwp) THEN
         WRITE(numout,*)
         WRITE(numout,*) 'trd_icp_init : integral constraints properties trends'
         WRITE(numout,*) '~~~~~~~~~~~~~'
      ENDIF

      ! Total volume at t-points:
      tvolt = 0.e0
      zsurf(:,:) = e1t(:,:) * e2t(:,:) * tmask_i(:,:)
      DO jk = 1, jpkm1
            tvolt = tvolt + SUM( tmask(:,:,jk) * zsurf(:,:) * fse3t(:,:,jk) )
      END DO
      IF( lk_mpp )   CALL mpp_sum( tvolt )   ! sum over the global domain

      IF(lwp) WRITE(numout,*) '                total ocean volume at T-point   tvolt = ',tvolt

#if  defined key_trddyn
      ! Initialization of potential to kinetic energy conversion
      rpktrd = 0.e0

      ! Total volume at u-, v- points:
      tvolu = 0.e0
      tvolv = 0.e0

      DO jk = 1, jpk
         DO jj = 2, jpjm1
            DO ji = fs_2, fs_jpim1   ! vector opt.
               zmsku = tmask_i(ji+1,jj  ) * tmask_i(ji,jj) * umask(ji,jj,jk)
               zmskv = tmask_i(ji  ,jj+1) * tmask_i(ji,jj) * vmask(ji,jj,jk)
               tvolu = tvolu + zmsku * e1u(ji,jj) * e2u(ji,jj) * fse3u(ji,jj,jk)
               tvolv = tvolv + zmskv * e1v(ji,jj) * e2v(ji,jj) * fse3v(ji,jj,jk)
            END DO
         END DO
      END DO
      IF( lk_mpp )   CALL mpp_sum( tvolu )   ! sums over the global domain
      IF( lk_mpp )   CALL mpp_sum( tvolv )

      IF(lwp) THEN
         WRITE(numout,*) '                total ocean volume at U-point   tvolu = ',tvolu
         WRITE(numout,*) '                total ocean volume at V-point   tvolv = ',tvolv
      ENDIF
#endif
      !
   END SUBROUTINE trd_icp_init


   SUBROUTINE trd_dwr( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_dwr  ***
      !! 
      !! ** Purpose :  write dynamic trends in ocean.output 
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt                                  ! ocean time-step index
      !!
      INTEGER  ::   ji, jj, jk
      REAL(wp) ::   zcof, zbe1ru, zbe2rv, zbtr, ztz, zth   !    "      scalars
      REAL(wp), DIMENSION(jpi,jpj)     ::   zsurf          ! 2D workspace array
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zkepe, zkx, zky, zkz   ! temporary arrays
      !!----------------------------------------------------------------------

      ! I. Momentum trends
      ! -------------------

      IF( MOD(kt,ntrd) == 0 .OR. kt == nit000 .OR. kt == nitend ) THEN

         ! I.1 Conversion potential energy - kinetic energy
         ! --------------------------------------------------
         ! c a u t i o n here, trends are computed at kt+1 (now , but after the swap)

         zkx(:,:,:)   = 0.e0
         zky(:,:,:)   = 0.e0
         zkz(:,:,:)   = 0.e0
         zkepe(:,:,:) = 0.e0
   
         CALL eos( tn, sn, rhd, rhop )       ! now potential and in situ densities

         ! Density flux at w-point
         zsurf(:,:) = 0.5 * e1t(:,:) * e2t(:,:) * tmask_i(:,:) /rau0
!!gm better use bn2....
         zkz(:,:,1) = 0.e0
         DO jk = 2, jpk
            zkz(:,:,jk) = zsurf(:,:) * wn(:,:,jk) * ( rhop(:,:,jk) + rhop(:,:,jk-1) )
         END DO
         
         ! Density flux at u and v-points
         DO jk = 1, jpk
            DO jj = 1, jpjm1
               DO ji = 1, jpim1
                  zcof   = 0.5 / rau0
                  zbe1ru = zcof * e2u(ji,jj) * fse3u(ji,jj,jk) * un(ji,jj,jk)
                  zbe2rv = zcof * e1v(ji,jj) * fse3v(ji,jj,jk) * vn(ji,jj,jk)
                  zkx(ji,jj,jk) = zbe1ru * ( rhop(ji,jj,jk) + rhop(ji+1,jj,jk) )
                  zky(ji,jj,jk) = zbe2rv * ( rhop(ji,jj,jk) + rhop(ji,jj+1,jk) )
               END DO
            END DO
         END DO
         
         ! Density flux divergence at t-point
         DO jk = 1, jpkm1
            DO jj = 2, jpjm1
               DO ji = 2, jpim1
                  zbtr = 1. / ( e1t(ji,jj)*e2t(ji,jj)*fse3t(ji,jj,jk) )
                  ztz = - zbtr * (    zkz(ji,jj,jk) - zkz(ji,jj,jk+1) )
                  zth = - zbtr * (  ( zkx(ji,jj,jk) - zkx(ji-1,jj,jk) )   &
                    &             + ( zky(ji,jj,jk) - zky(ji,jj-1,jk) )  )
                  zkepe(ji,jj,jk) = (zth + ztz) * tmask(ji,jj,jk) * tmask_i(ji,jj)
               END DO
            END DO
         END DO
         zkepe( : , : ,jpk) = 0.e0
         zkepe( : ,jpj, : ) = 0.e0
         zkepe(jpi, : , : ) = 0.e0

         ! I.2 Basin averaged kinetic energy trend
         ! ----------------------------------------
         peke = 0.e0
         DO jk = 1,jpk
            DO jj = 1, jpj
               DO ji = 1, jpi
                  peke = peke + zkepe(ji,jj,jk) * grav * fsdept(ji,jj,jk)   &
                     &                     * e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk)
               END DO
            END DO
         END DO

         ! I.3 Sums over the global domain
         ! ---------------------------------
         IF( lk_mpp ) THEN
            CALL mpp_sum( peke )
            CALL mpp_sum( umo , jptot_dyn )
            CALL mpp_sum( vmo , jptot_dyn )
            CALL mpp_sum( hke , jptot_dyn )
         ENDIF

         ! I.2 Print dynamic trends in the ocean.output file
         ! --------------------------------------------------

         IF(lwp) THEN
            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9500) kt
            WRITE (numout,9501) umo(jpicpd_hpg) / tvolu, vmo(jpicpd_hpg) / tvolv
            WRITE (numout,9502) umo(jpicpd_keg) / tvolu, vmo(jpicpd_keg) / tvolv
            WRITE (numout,9503) umo(jpicpd_rvo) / tvolu, vmo(jpicpd_rvo) / tvolv
            WRITE (numout,9504) umo(jpicpd_pvo) / tvolu, vmo(jpicpd_pvo) / tvolv
            WRITE (numout,9505) umo(jpicpd_ldf) / tvolu, vmo(jpicpd_ldf) / tvolv
            WRITE (numout,9506) umo(jpicpd_zad) / tvolu, vmo(jpicpd_zad) / tvolv
            WRITE (numout,9507) umo(jpicpd_zdf) / tvolu, vmo(jpicpd_zdf) / tvolv
            WRITE (numout,9508) umo(jpicpd_spg) / tvolu, vmo(jpicpd_spg) / tvolv
            WRITE (numout,9509) umo(jpicpd_swf) / tvolu, vmo(jpicpd_swf) / tvolv
            WRITE (numout,9510) umo(jpicpd_dat) / tvolu, vmo(jpicpd_dat) / tvolv
            WRITE (numout,9511) umo(jpicpd_bfr) / tvolu, vmo(jpicpd_bfr) / tvolv
            WRITE (numout,9512)
            WRITE (numout,9513)                                                 &
            &     (  umo(jpicpd_hpg) + umo(jpicpd_keg) + umo(jpicpd_rvo) + umo(jpicpd_pvo) + umo(jpicpd_ldf)   &
            &      + umo(jpicpd_zad) + umo(jpicpd_zdf) + umo(jpicpd_spg) + umo(jpicpd_dat) + umo(jpicpd_swf)   &
            &      + umo(jpicpd_bfr) ) / tvolu,   &
            &     (  vmo(jpicpd_hpg) + vmo(jpicpd_keg) + vmo(jpicpd_rvo) + vmo(jpicpd_pvo) + vmo(jpicpd_ldf)   &
            &      + vmo(jpicpd_zad) + vmo(jpicpd_zdf) + vmo(jpicpd_spg) + vmo(jpicpd_dat) + vmo(jpicpd_swf)   &
            &      + vmo(jpicpd_bfr) ) / tvolv
         ENDIF

 9500    FORMAT(' momentum trend at it= ', i6, ' :', /' ==============================')
 9501    FORMAT(' pressure gradient          u= ', e20.13, '    v= ', e20.13)
 9502    FORMAT(' ke gradient                u= ', e20.13, '    v= ', e20.13)
 9503    FORMAT(' relative vorticity term    u= ', e20.13, '    v= ', e20.13)
 9504    FORMAT(' coriolis term              u= ', e20.13, '    v= ', e20.13)
 9505    FORMAT(' horizontal diffusion       u= ', e20.13, '    v= ', e20.13)
 9506    FORMAT(' vertical advection         u= ', e20.13, '    v= ', e20.13)
 9507    FORMAT(' vertical diffusion         u= ', e20.13, '    v= ', e20.13)
 9508    FORMAT(' surface pressure gradient  u= ', e20.13, '    v= ', e20.13)
 9509    FORMAT(' surface wind forcing       u= ', e20.13, '    v= ', e20.13)
 9510    FORMAT(' dampimg term               u= ', e20.13, '    v= ', e20.13)
 9511    FORMAT(' bottom flux                u= ', e20.13, '    v= ', e20.13)
 9512    FORMAT(' -----------------------------------------------------------------------------')
 9513    FORMAT(' total trend                u= ', e20.13, '    v= ', e20.13)

         IF(lwp) THEN
            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9520) kt
            WRITE (numout,9521) hke(jpicpd_hpg) / tvolt
            WRITE (numout,9522) hke(jpicpd_keg) / tvolt
            WRITE (numout,9523) hke(jpicpd_rvo) / tvolt
            WRITE (numout,9524) hke(jpicpd_pvo) / tvolt
            WRITE (numout,9525) hke(jpicpd_ldf) / tvolt
            WRITE (numout,9526) hke(jpicpd_zad) / tvolt
            WRITE (numout,9527) hke(jpicpd_zdf) / tvolt
            WRITE (numout,9528) hke(jpicpd_spg) / tvolt
            WRITE (numout,9529) hke(jpicpd_swf) / tvolt
            WRITE (numout,9530) hke(jpicpd_dat) / tvolt
            WRITE (numout,9531)
            WRITE (numout,9532)   &
            &     (  hke(jpicpd_hpg) + hke(jpicpd_keg) + hke(jpicpd_rvo) + hke(jpicpd_pvo) + hke(jpicpd_ldf)   &
            &      + hke(jpicpd_zad) + hke(jpicpd_zdf) + hke(jpicpd_spg) + hke(jpicpd_dat) + hke(jpicpd_swf) ) / tvolt
         ENDIF

 9520    FORMAT(' kinetic energy trend at it= ', i6, ' :', /' ====================================')
 9521    FORMAT(' pressure gradient         u2= ', e20.13)
 9522    FORMAT(' ke gradient               u2= ', e20.13)
 9523    FORMAT(' relative vorticity term   u2= ', e20.13)
 9524    FORMAT(' coriolis term             u2= ', e20.13)
 9525    FORMAT(' horizontal diffusion      u2= ', e20.13)
 9526    FORMAT(' vertical advection        u2= ', e20.13)
 9527    FORMAT(' vertical diffusion        u2= ', e20.13)
 9528    FORMAT(' surface pressure gradient u2= ', e20.13)
 9529    FORMAT(' surface wind forcing      u2= ', e20.13)
 9530    FORMAT(' dampimg term              u2= ', e20.13)
 9531    FORMAT(' --------------------------------------------------')
 9532    FORMAT(' total trend               u2= ', e20.13)

         IF(lwp) THEN
            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9540) kt
            WRITE (numout,9541) ( hke(jpicpd_keg) + hke(jpicpd_rvo) + hke(jpicpd_zad) ) / tvolt
            WRITE (numout,9542) ( hke(jpicpd_keg) + hke(jpicpd_zad) ) / tvolt
            WRITE (numout,9543) ( hke(jpicpd_pvo) ) / tvolt
            WRITE (numout,9544) ( hke(jpicpd_rvo) ) / tvolt
            WRITE (numout,9545) ( hke(jpicpd_spg) ) / tvolt
            WRITE (numout,9546) ( hke(jpicpd_ldf) ) / tvolt
            WRITE (numout,9547) ( hke(jpicpd_zdf) ) / tvolt
            WRITE (numout,9548) ( hke(jpicpd_hpg) ) / tvolt, rpktrd / tvolt
            WRITE (numout,*)
            WRITE (numout,*)
         ENDIF

 9540    FORMAT(' energetic consistency at it= ', i6, ' :', /' =========================================')
 9541    FORMAT(' 0 = non linear term(true if key_vorenergy or key_combined): ', e20.13)
 9542    FORMAT(' 0 = ke gradient + vertical advection                      : ', e20.13)
 9543    FORMAT(' 0 = coriolis term  (true if key_vorenergy or key_combined): ', e20.13)
 9544    FORMAT(' 0 = uh.( rot(u) x uh ) (true if enstrophy conser.)        : ', e20.13)
 9545    FORMAT(' 0 = surface pressure gradient                             : ', e20.13)
 9546    FORMAT(' 0 > horizontal diffusion                                  : ', e20.13)
 9547    FORMAT(' 0 > vertical diffusion                                    : ', e20.13)
 9548    FORMAT(' pressure gradient u2 = - 1/rau0 u.dz(rhop)                : ', e20.13, '  u.dz(rhop) =', e20.13)
         !
         ! Save potential to kinetic energy conversion for next time step
         rpktrd = peke
         !
      ENDIF
      !
   END SUBROUTINE trd_dwr


   SUBROUTINE trd_twr( kt )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_twr  ***
      !! 
      !! ** Purpose :  write active tracers trends in ocean.output 
      !!----------------------------------------------------------------------
      INTEGER, INTENT(in) ::   kt   ! ocean time-step index
      !!----------------------------------------------------------------------

      ! I. Tracers trends
      ! -----------------

      IF( MOD(kt,ntrd) == 0 .OR. kt == nit000 .OR. kt == nitend ) THEN

         ! I.1 Sums over the global domain
         ! -------------------------------
         IF( lk_mpp ) THEN
            CALL mpp_sum( tsmo, jpt_trd )   
            CALL mpp_sum( ts2 , jpt_trd )
         ENDIF

         ! I.2 Print tracers trends in the ocean.output file
         ! --------------------------------------------------
         
         IF(lwp) THEN
            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9400) kt
            WRITE (numout,9401) ( tsmo(jpt_trd_xad,jp_tem) + tsmo(jpt_trd_yad,jp_tem)  ) / tvolt,   &
               &                ( tsmo(jpt_trd_xad,jp_sal) + tsmo(jpt_trd_yad,jp_sal)  ) / tvolt
            WRITE (numout,9402)   tsmo(jpt_trd_zad,jp_tem) / tvolt,   tsmo(jpt_trd_zad,jp_sal) / tvolt
            WRITE (numout,9403)   tsmo(jpt_trd_ldf,jp_tem) / tvolt,   tsmo(jpt_trd_ldf,jp_sal) / tvolt
            WRITE (numout,9404)   tsmo(jpt_trd_zdf,jp_tem) / tvolt,   tsmo(jpt_trd_zdf,jp_sal) / tvolt
            WRITE (numout,9405)   tsmo(jpt_trd_npc,jp_tem) / tvolt,   tsmo(jpt_trd_npc,jp_sal) / tvolt
            WRITE (numout,9406)   tsmo(jpt_trd_dmp,jp_tem) / tvolt,   tsmo(jpt_trd_dmp,jp_sal) / tvolt
            WRITE (numout,9407)   tsmo(jpt_trd_qsr,jp_tem) / tvolt
            WRITE (numout,9408)   tsmo(jpt_trd_qns,jp_tem) / tvolt,   tsmo(jpt_trd_qns,jp_sal) / tvolt
            WRITE (numout,9409) 
            WRITE (numout,9410) (  tsmo(jpt_trd_xad,jp_tem) + tsmo(jpt_trd_yad,jp_tem) + tsmo(jpt_trd_zad,jp_tem)     &
               &                 + tsmo(jpt_trd_ldf,jp_tem) + tsmo(jpt_trd_zdf,jp_tem) + tsmo(jpt_trd_npc,jp_tem)     &
               &                 + tsmo(jpt_trd_dmp,jp_tem) + tsmo(jpt_trd_qsr,jp_tem) + tsmo(jpt_trd_qns,jp_tem) )   &
               &                / tvolt,   &
               &                (  tsmo(jpt_trd_xad,jp_sal) + tsmo(jpt_trd_yad,jp_sal) + tsmo(jpt_trd_zad,jp_sal)     &
               &                 + tsmo(jpt_trd_ldf,jp_sal) + tsmo(jpt_trd_zdf,jp_sal) + tsmo(jpt_trd_npc,jp_sal)     &
               &                 + tsmo(jpt_trd_dmp,jp_sal)                           + tsmo(jpt_trd_qns,jp_sal) )   &
               &                / tvolt


9400     FORMAT(' tracer trend at it= ',i6,' :     temperature              salinity', /,   &
                ' ============================')
9401     FORMAT(' horizontal advection        ',e20.13,'     ',e20.13)
9402     FORMAT(' vertical advection          ',e20.13,'     ',e20.13)
9403     FORMAT(' horizontal diffusion        ',e20.13,'     ',e20.13)
9404     FORMAT(' vertical diffusion          ',e20.13,'     ',e20.13)
9405     FORMAT(' static instability mixing   ',e20.13,'     ',e20.13)
9406     FORMAT(' damping term                ',e20.13,'     ',e20.13)
9407     FORMAT(' penetrative qsr             ',e20.13)
9408     FORMAT(' non solar radiation         ',e20.13,'     ',e20.13)
9409     FORMAT(' -------------------------------------------------------------------------')
9410     FORMAT(' total trend                 ',e20.13,'     ',e20.13)


            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9420) kt
            WRITE (numout,9421) ( ts2(jpt_trd_xad,jp_tem) + ts2(jpt_trd_yad,jp_tem) ) / tvolt,   &
               &                ( ts2(jpt_trd_xad,jp_sal) + ts2(jpt_trd_yad,jp_sal) ) / tvolt
            WRITE (numout,9422)   ts2(jpt_trd_zad,jp_tem) / tvolt, ts2(jpt_trd_zad,jp_sal) / tvolt
            WRITE (numout,9423)   ts2(jpt_trd_ldf,jp_tem) / tvolt, ts2(jpt_trd_ldf,jp_sal) / tvolt
            WRITE (numout,9424)   ts2(jpt_trd_zdf,jp_tem) / tvolt, ts2(jpt_trd_zdf,jp_sal) / tvolt
            WRITE (numout,9425)   ts2(jpt_trd_npc,jp_tem) / tvolt, ts2(jpt_trd_npc,jp_sal) / tvolt
            WRITE (numout,9426)   ts2(jpt_trd_dmp,jp_tem) / tvolt, ts2(jpt_trd_dmp,jp_sal) / tvolt
            WRITE (numout,9427)   ts2(jpt_trd_qsr,jp_tem) / tvolt
            WRITE (numout,9428)   ts2(jpt_trd_qns,jp_tem) / tvolt, ts2(jpt_trd_qns,jp_sal) / tvolt
            WRITE (numout,9429)
            WRITE (numout,9430) (  ts2(jpt_trd_xad,jp_tem) + ts2(jpt_trd_yad,jp_tem) + ts2(jpt_trd_zad,jp_tem)     &
               &                 + ts2(jpt_trd_ldf,jp_tem) + ts2(jpt_trd_zdf,jp_tem) + ts2(jpt_trd_npc,jp_tem)     &
               &                 + ts2(jpt_trd_dmp,jp_tem) + ts2(jpt_trd_qsr,jp_tem) + ts2(jpt_trd_qns,jp_tem) )   &
               &                / tvolt,   &
               &                (  ts2(jpt_trd_xad,jp_sal) + ts2(jpt_trd_yad,jp_sal) + ts2(jpt_trd_zad,jp_sal)     &
               &                 + ts2(jpt_trd_ldf,jp_sal) + ts2(jpt_trd_zdf,jp_sal) + ts2(jpt_trd_npc,jp_sal)     &
               &                 + ts2(jpt_trd_dmp,jp_sal)                          + ts2(jpt_trd_qns,jp_sal) )   &
               &                / tvolt

9420     FORMAT(' tracer**2 trend at it= ',i6,' :     temperature              salinity', /,   &
                ' ============================')
9421     FORMAT(' horizontal advection      * t   ', e20.13, '     ', e20.13)
9422     FORMAT(' vertical advection        * t   ', e20.13, '     ', e20.13)
9423     FORMAT(' horizontal diffusion      * t   ', e20.13, '     ', e20.13)
9424     FORMAT(' vertical diffusion        * t   ', e20.13, '     ', e20.13)
9425     FORMAT(' static instability mixing * t   ', e20.13, '     ', e20.13)
9426     FORMAT(' damping term              * t   ', e20.13, '     ', e20.13)
9427     FORMAT(' penetrative qsr           * t   ', e20.13)
9428     FORMAT(' non solar radiation       * t   ', e20.13, '     ', e20.13)
9429     FORMAT(' -----------------------------------------------------------------------------')
9430     FORMAT(' total trend                *t = ', e20.13, '  *s = ', e20.13)


            WRITE (numout,*)
            WRITE (numout,*)
            WRITE (numout,9440) kt
            WRITE (numout,9441) ( tsmo(jpt_trd_xad,jp_tem)+tsmo(jpt_trd_yad,jp_tem)+tsmo(jpt_trd_zad,jp_tem) )/tvolt,  &
            &                   ( tsmo(jpt_trd_xad,jp_sal)+tsmo(jpt_trd_yad,jp_sal)+tsmo(jpt_trd_zad,jp_sal) )/tvolt
            WRITE (numout,9442)   tsmo(jpt_trd_zl1,jp_tem)/tvolt,  tsmo(jpt_trd_zl1,jp_sal)/tvolt
            WRITE (numout,9443)   tsmo(jpt_trd_ldf,jp_tem)/tvolt,  tsmo(jpt_trd_ldf,jp_sal)/tvolt
            WRITE (numout,9444)   tsmo(jpt_trd_zdf,jp_tem)/tvolt,  tsmo(jpt_trd_zdf,jp_sal)/tvolt
            WRITE (numout,9445)   tsmo(jpt_trd_npc,jp_tem)/tvolt,  tsmo(jpt_trd_npc,jp_sal)/tvolt
            WRITE (numout,9446) ( ts2(jpt_trd_xad,jp_tem)+ts2(jpt_trd_yad,jp_tem)+ts2(jpt_trd_zad,jp_tem) )/tvolt,    &
            &                   ( ts2(jpt_trd_xad,jp_sal)+ts2(jpt_trd_yad,jp_sal)+ts2(jpt_trd_zad,jp_sal) )/tvolt
            WRITE (numout,9447)   ts2(jpt_trd_ldf,jp_tem)/tvolt,   ts2(jpt_trd_ldf,jp_sal)/tvolt
            WRITE (numout,9448)   ts2(jpt_trd_zdf,jp_tem)/tvolt,   ts2(jpt_trd_zdf,jp_sal)/tvolt
            WRITE (numout,9449)   ts2(jpt_trd_npc,jp_tem)/tvolt,   ts2(jpt_trd_npc,jp_sal)/tvolt
         ENDIF

9440     FORMAT(' tracer consistency at it= ',i6, ' :         temperature','                salinity',   &
                /, ' ==================================')
9441     FORMAT(' 0 = horizontal+vertical advection +    ',e20.13,'       ',e20.13)
9442     FORMAT('     1st lev vertical advection         ',e20.13,'       ',e20.13)
9443     FORMAT(' 0 = horizontal diffusion               ',e20.13,'       ',e20.13)
9444     FORMAT(' 0 = vertical diffusion                 ',e20.13,'       ',e20.13)
9445     FORMAT(' 0 = static instability mixing          ',e20.13,'       ',e20.13)
9446     FORMAT(' 0 = horizontal+vertical advection * t  ',e20.13,'       ',e20.13)
9447     FORMAT(' 0 > horizontal diffusion          * t  ',e20.13,'       ',e20.13)
9448     FORMAT(' 0 > vertical diffusion            * t  ',e20.13,'       ',e20.13)
9449     FORMAT(' 0 > static instability mixing     * t  ',e20.13,'       ',e20.13)
         !
      ENDIF
      !
   END SUBROUTINE trd_twr

#   else
   !!----------------------------------------------------------------------
   !!   Default case :                                         Empty module
   !!----------------------------------------------------------------------
   INTERFACE trd_icp
      MODULE PROCEDURE trd_2d, trd_3d
   END INTERFACE

CONTAINS
   SUBROUTINE trd_2d( ptrd2dx, ptrd2dy, ktrd , ctype, clpas )       ! Empty routine
      REAL, DIMENSION(:,:) ::   ptrd2dx, ptrd2dy
      INTEGER                     , INTENT(in   ) ::   ktrd         ! tracer trend index
      CHARACTER(len=3)            , INTENT(in   ) ::   ctype        ! momentum ('DYN') or tracers ('TRA') trends
      CHARACTER(len=3), INTENT(in), OPTIONAL ::   clpas             ! number of passage
      WRITE(*,*) 'trd_2d: You should not have seen this print! error ?', &
          &       ptrd2dx(1,1), ptrd2dy(1,1), ktrd, ctype, clpas
   END SUBROUTINE trd_2d
   SUBROUTINE trd_3d( ptrd3dx, ptrd3dy, ktrd , ctype, clpas )       ! Empty routine
      REAL, DIMENSION(:,:,:) ::   ptrd3dx, ptrd3dy
      INTEGER                     , INTENT(in   ) ::   ktrd         ! tracer trend index
      CHARACTER(len=3)            , INTENT(in   ) ::   ctype        ! momentum ('DYN') or tracers ('TRA') trends
      CHARACTER(len=3), INTENT(in), OPTIONAL ::   clpas             ! number of passage
      WRITE(*,*) 'trd_3d: You should not have seen this print! error ?', &
          &       ptrd3dx(1,1,1), ptrd3dy(1,1,1), ktrd, ctype, clpas
   END SUBROUTINE trd_3d
   SUBROUTINE trd_icp_init               ! Empty routine
   END SUBROUTINE trd_icp_init
   SUBROUTINE trd_dwr( kt )          ! Empty routine
      WRITE(*,*) 'trd_dwr: You should not have seen this print! error ?', kt
   END SUBROUTINE trd_dwr
   SUBROUTINE trd_twr( kt )          ! Empty routine
      WRITE(*,*) 'trd_twr: You should not have seen this print! error ?', kt
   END SUBROUTINE trd_twr
#endif

   !!======================================================================
END MODULE trdicp
