MODULE trdmod
   !!======================================================================
   !!                       ***  MODULE  trdmod  ***
   !! Ocean diagnostics:  ocean tracers and dynamic trends
   !!=====================================================================
   !! History :  9.0  !  04-08  (C. Talandier) Original code
   !!                 !  05-04  (C. Deltel)    Add Asselin trend in the ML budget
   !!----------------------------------------------------------------------
#if  defined key_trdtra || defined key_trddyn || defined key_trdmld || defined key_trdvor || defined key_esopa
   !!----------------------------------------------------------------------
   !!   trd_tra          : active tracer trend manager
   !!   trd_tra_adv      : pre-treatment of the tracer advection trends
   !!   trd_mod          : momentum trend manager
   !!   trd_mod_init     : Initialization step
   !!----------------------------------------------------------------------
   USE phycst                  ! physical constants
   USE oce                     ! ocean dynamics and tracers variables
   USE dom_oce                 ! ocean space and time domain variables
   USE zdf_oce                 ! ocean vertical physics variables
   USE trdmod_oce              ! ocean variables trends
   USE ldftra_oce              ! ocean active tracers lateral physics
   USE trdvor                  ! ocean vorticity trends 
   USE trdicp                  ! ocean bassin integral constraints properties
   USE trdmld                  ! ocean active mixed layer tracers trends 
   USE in_out_manager          ! I/O manager
   USE taumod                  ! surface ocean stress

   IMPLICIT NONE
   PRIVATE

   REAL(wp) ::   r2dt          ! time-step, = 2 rdttra except at nit000 (=rdttra) if neuler=0

   PUBLIC trd_tra              ! called by all traXXX.F90 modules
   PUBLIC trd_tra_adv          ! called by all traadv_XXX.F90 modules
   PUBLIC trd_mod              ! called by all dynXXX.F90 modules
   PUBLIC trd_mod_init         ! called by opa.F90 module

   !! * Substitutions
#  include "domzgr_substitute.h90"
#  include "vectopt_loop_substitute.h90"
   !!----------------------------------------------------------------------
   !!   OPA 9.0 , LOCEAN-IPSL (2005) 
   !! $Header$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE trd_tra( kt, ktra, ktrd, ctype, ptrd2d, ptrd3d, cnbpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_mod  ***
      !! 
      !! ** Purpose : Dispatch all trends computation, e.g. vorticity, mld or 
      !!              integral constraints
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in   )                                   ::   kt      ! time step
      INTEGER         , INTENT(in   )                                   ::   ktra    ! tracer index
      INTEGER         , INTENT(in   )                                   ::   ktrd    ! tracer trend index
      CHARACTER(len=3), INTENT(in   )                                   ::   ctype   ! tracers type 'TRA' or 'TRC'
      REAL(wp)        , INTENT(inout), DIMENSION(jpi,jpj)    , OPTIONAL ::   ptrd2d  ! Temperature or U trend 
      REAL(wp)        , INTENT(inout), DIMENSION(jpi,jpj,jpk), OPTIONAL ::   ptrd3d  ! Temperature or U trend 
      CHARACTER(len=3), INTENT(in   )                        , OPTIONAL ::   cnbpas  ! number of passage
      !!
      CHARACTER(len=3) ::   ccpas                                  ! number of passage
!     REAL(wp), DIMENSION(jpi,jpj) ::   z2dx, z2dy                 ! workspace arrays
      !!----------------------------------------------------------------------

      ccpas = 'fst'                            ! Control of optional arguments
      IF( PRESENT(cnbpas) )  ccpas = cnbpas

      IF( neuler == 0 .AND. kt == nit000    ) THEN   ;   r2dt =      rdt      ! = rdtra (restarting with Euler time stepping)
      ELSEIF(               kt <= nit000 + 1) THEN   ;   r2dt = 2. * rdt      ! = 2 rdttra (leapfrog)
      ENDIF

      !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      ! I. Integral Constraints Properties for momentum and/or tracers trends
      !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      IF( ( mod(kt,ntrd) == 0 .OR. kt == nit000 .OR. kt == nitend) )   THEN
         !
         IF( lk_trdtra .AND. ctype == 'TRA' )   THEN       ! active tracer trends
            IF( PRESENT(ptrd2d) ) THEN   ;   CALL trd_icp( ptrd2d, ktra, ktrd, ctype, clpas=ccpas )
            ELSE                         ;   CALL trd_icp( ptrd3d, ktra, ktrd, ctype, clpas=ccpas )
            ENDIF
         ENDIF
!           SELECT CASE ( ktrd )
!           CASE ( jptra_trd_ldf )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_ldf, ctype )   ! lateral diff
!           CASE ( jptra_trd_zdf )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_zdf, ctype )   ! vertical diff (Kz)
!           CASE ( jptra_trd_bbc )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_bbc, ctype )   ! bottom boundary cond
!           CASE ( jptra_trd_bbl )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_bbl, ctype )   ! bottom boundary layer
!           CASE ( jptra_trd_npc )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_npc, ctype )   ! static instability mixing
!           CASE ( jptra_trd_dmp )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_dmp, ctype )   ! damping
!           CASE ( jptra_trd_qsr )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_qsr, ctype )   ! penetrative solar radiat.
!           CASE ( jptra_trd_nsr )   
!              z2dx(:,:) = ptrdx(:,:,1)   ;   z2dy(:,:) = ptrdy(:,:,1)
!              CALL trd_icp( z2dx, z2dy, jpicpt_nsr, ctype )                               ! non solar radiation
!           CASE ( jptra_trd_xad )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_xad, ctype )   ! x- horiz adv
!           CASE ( jptra_trd_yad )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_yad, ctype )   ! y- horiz adv
!           CASE ( jptra_trd_zad )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpt_zad, ctype, clpas=ccpas )    ! z- adv 
!              CALL trd_icp( ptrdx, ptrdy, jpicpt_zad, ctype, clpas=ccpas )   
!              ! compute the surface flux condition wn(:,:,1)*tn(:,:,1)
!              z2dx(:,:) = wn(:,:,1)*tn(:,:,1)/fse3t(:,:,1)
!              z2dy(:,:) = wn(:,:,1)*sn(:,:,1)/fse3t(:,:,1)
!              CALL trd_icp( z2dx , z2dy , jpicpt_zl1, ctype )                             ! 1st z- vertical adv 
!           END SELECT
!        END IF
         !
      END IF

      !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      ! III. Mixed layer trends for active tracers
      !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      IF( lk_trdmld .AND. ctype == 'TRA' )   THEN
         
         !-----------------------------------------------------------------------------------------------
         ! W.A.R.N.I.N.G :
         ! jptra_trd_ldf : called by traldf.F90
         !                 at this stage we store:
         !                  - the lateral geopotential diffusion (here, lateral = horizontal)
         !                  - and the iso-neutral diffusion if activated 
         ! jptra_trd_zdf : called by trazdf.F90
         !                 * in case of iso-neutral diffusion we store the vertical diffusion component in the 
         !                   lateral trend including the K_z contrib, which will be removed later (see trd_mld)
         !-----------------------------------------------------------------------------------------------

!        SELECT CASE ( ktrd )
!        CASE ( jpt_trd_zdf )
!           IF( ln_traldf_iso )   THEN
!              CALL trd_mld_zint( ptrdx, ptrdy, jpmld_ldf, '3D' )   ! vertical diffusion (K_z)
!           ELSE
!              CALL trd_mld_zint( ptrdx, ptrdy, jpmld_zdf, '3D' )   ! vertical diffusion (K_z)
!           ENDIF
!        CASE ( jpt_trd_nsr )
!           ptrdx(:,:,2:jpk) = 0.e0   ;   ptrdy(:,:,2:jpk) = 0.e0
!           CALL trd_mld_zint( ptrdx, ptrdy, jpmld_for, '2D' )                             ! air-sea : non solar flux
!        CASE ( jpt_trd_bbc )   ;   CALL trd_mld_zint( ptrdx, ptrdy, jpmld_bbc, '3D' )   ! bbc (geothermal flux)
!        CASE DEFAULT
!                                     CALL trd_mld_zint( ptrdx, ptrdy, ktra     , '3D' )   ! other 3D trends
!        END SELECT

      ENDIF

   END SUBROUTINE trd_tra


   SUBROUTINE trd_tra_adv( kt, ktra, ktrd, ctype, pf, pun, ptn, cnbpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_mod  ***
      !! 
      !! ** Purpose :   transformed the i-advective flux into the i-advective trends
      !! ** Method  :   i-advective trends = un. di[T] = di[fi] - tn di[un]
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in   )                         ::   kt      ! time step
      INTEGER         , INTENT(in   )                         ::   ktra    ! tracer index
      INTEGER         , INTENT(in   )                         ::   ktrd    ! tracer trend index
      CHARACTER(len=3), INTENT(in   )                         ::   ctype   ! tracers type 'TRA' or 'TRC'
      REAL(wp)        , INTENT(in   ), DIMENSION(jpi,jpj,jpk) ::   pf      ! advective flux in one direction
      REAL(wp)        , INTENT(in   ), DIMENSION(jpi,jpj,jpk) ::   pun     ! now velocity  in one direction
      REAL(wp)        , INTENT(in   ), DIMENSION(jpi,jpj,jpk) ::   ptn     ! now or before tracer 
      CHARACTER(len=3), INTENT(in   )                        , OPTIONAL ::   cnbpas  ! number of passage
      !!
      INTEGER                          ::   ji, jj, jk      ! dummy loop indices
      CHARACTER(len=3) ::   ccpas                           ! number of passage
      REAL(wp)                         ::   zbtr, z_hdivn   ! 
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   ztrdt           ! 3D workspace 
      !!----------------------------------------------------------------------

      ccpas = 'fst'                            ! Control of optional arguments
      IF( PRESENT(cnbpas) )  ccpas = cnbpas

      ztrdt(:,:,:) = 0.e0
      !
      IF( ccpas == 'fst' ) THEN      ! first treatment : remove the divergence
         SELECT CASE( ktrd ) 
         CASE( jpt_trd_xad )      ! i-advective trend
            DO jk = 1, jpkm1
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
#if defined key_zco
                     zbtr    = 1.e0 / ( e1t(ji,jj) * e2t(ji,jj) )
                     z_hdivn = (  e2u(ji  ,jj) * pun(ji  ,jj,jk)                              &
                        &       - e2u(ji-1,jj) * pun(ji-1,jj,jk) )
#else
                     zbtr    = 1.e0/ ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
                     z_hdivn = (  e2u(ji  ,jj) * fse3u(ji  ,jj,jk) * pun(ji  ,jj,jk)          &
                        &       - e2u(ji-1,jj) * fse3u(ji-1,jj,jk) * pun(ji-1,jj,jk) )
#endif
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji-1,jj,jk)  -  ptn(ji,jj,jk) * z_hdivn  )
                  END DO
               END DO
            END DO
            !
         CASE( jpt_trd_yad )      ! j-advective trend
            DO jk = 1, jpkm1
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
#if defined key_zco
                     zbtr    = 1.e0 / ( e1t(ji,jj) * e2t(ji,jj) )
                     z_hdivn = (  e1v(ji,jj  ) * pun(ji,jj  ,jk)                              &
                        &       - e1v(ji,jj-1) * pun(ji,jj-1,jk) )
#else
                     zbtr    = 1.e0/ ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
                     z_hdivn = (  e1v(ji  ,jj) * fse3v(ji,jj  ,jk) * pun(ji,jj  ,jk)          &
                        &       - e1v(ji-1,jj) * fse3v(ji,jj-1,jk) * pun(ji,jj-1,jk) )
#endif
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji,jj-1,jk)  -  ptn(ji,jj,jk) * z_hdivn  )
                  END DO
               END DO
            END DO
            !
         CASE( jpt_trd_zad )      ! z-advective trend
            DO jk = 1, jpkm1
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
                     zbtr    = 1.e0 / fse3t(ji,jj,jk)
                     z_hdivn = pun(ji,jj,jk) - pun(ji,jj,jk+1)
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji,jj,jk+1)  -  ptn(ji,jj,jk) * z_hdivn  )
                  END DO
               END DO
            END DO
            !
         END SELECT
         !
      ELSE                ! second call : just compute the trend   (TVD scheme)
         SELECT CASE( ktrd ) 
         CASE( jpt_trd_xad )      ! i-advective trend
            DO jk = 1, jpkm1      
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
                     zbtr    = 1.e0 / ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji-1,jj,jk)  )
                  END DO
               END DO
            END DO
            !
         CASE( jpt_trd_yad )      ! j-advective trend
            DO jk = 1, jpkm1
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
                     zbtr    = 1.e0 / ( e1t(ji,jj) * e2t(ji,jj) * fse3t(ji,jj,jk) )
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji,jj-1,jk)  )
                  END DO
               END DO
            END DO
            !
         CASE( jpt_trd_zad )      ! z-advective trend
            DO jk = 1, jpkm1
               DO jj = 2, jpjm1
                  DO ji = fs_2, fs_jpim1   ! vector opt.
                     zbtr    = 1.e0 / fse3t(ji,jj,jk)
                     ztrdt(ji,jj,jk) = - zbtr * (  pf(ji,jj,jk) - pf(ji,jj,jk+1)  )
                  END DO
               END DO
            END DO
            !
         END SELECT
         !
      ENDIF
      !
      CALL trd_tra( kt, ktra, ktrd, ctype, ptrd3d=ztrdt)       ! trend diagnostics
      !
   END SUBROUTINE trd_tra_adv


   SUBROUTINE trd_mod( ptrdx, ptrdy, ktrd, ctype, kt, cnbpas )
      !!---------------------------------------------------------------------
      !!                  ***  ROUTINE trd_mod  ***
      !! 
      !! ** Purpose : Dispatch all trends computation, e.g. vorticity, mld or 
      !!              integral constraints
      !!----------------------------------------------------------------------
      INTEGER         , INTENT(in   )                                   ::   kt      ! time step
      INTEGER         , INTENT(in   )                                   ::   ktrd    ! tracer trend index
      CHARACTER(len=3), INTENT(in   )                                   ::   ctype   ! momentum trends type ='DYN'
      CHARACTER(len=3), INTENT(in   )                        , OPTIONAL ::   cnbpas  ! number of passage
      REAL(wp)        , INTENT(inout), DIMENSION(jpi,jpj,jpk)           ::   ptrdx   ! Temperature or U trend 
      REAL(wp)        , INTENT(inout), DIMENSION(jpi,jpj,jpk)           ::   ptrdy   ! Salinity    or V trend
      !!
      INTEGER ::   ji, ikbu, ikbum1
      INTEGER ::   jj, ikbv, ikbvm1
      CHARACTER(len=3) ::   ccpas                                  ! number of passage
      REAL(wp) ::   zua, zva                                       ! scalars
      REAL(wp), DIMENSION(jpi,jpj) ::   ztswu, ztswv               ! 2D workspace
      REAL(wp), DIMENSION(jpi,jpj) ::   ztbfu, ztbfv               ! 2D workspace
      REAL(wp), DIMENSION(jpi,jpj) ::   z2dx, z2dy                 ! workspace arrays
      !!----------------------------------------------------------------------

      z2dx(:,:)   = 0.e0   ;   z2dy(:,:)   = 0.e0                  ! initialization of workspace arrays

      ccpas = 'fst'                            ! Control of optional arguments
      IF( PRESENT(cnbpas) )  ccpas = cnbpas

      IF( neuler == 0 .AND. kt == nit000    ) THEN   ;   r2dt =      rdt      ! = rdtra (start with Euler time stepping)
      ELSEIF(               kt <= nit000 + 1) THEN   ;   r2dt = 2. * rdt      ! = 2 rdttra (leapfrog)
      ENDIF

      !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      ! I. Integral Constraints Properties for momentum and/or tracers trends
      !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      IF( ( mod(kt,ntrd) == 0 .OR. kt == nit000 .OR. kt == nitend) )   THEN
         !
         IF( lk_trddyn .AND. ctype == 'DYN' )   THEN       ! momentum trends 
            !
            SELECT CASE ( ktrd )
            CASE ( jpdyn_trd_hpg )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_hpg, ctype )   ! hydrost. pressure grad
            CASE ( jpdyn_trd_keg )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_keg, ctype )   ! KE gradient 
            CASE ( jpdyn_trd_rvo )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_rvo, ctype )   ! relative vorticity 
            CASE ( jpdyn_trd_pvo )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_pvo, ctype )   ! planetary vorticity
            CASE ( jpdyn_trd_ldf )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_ldf, ctype )   ! lateral diffusion 
            CASE ( jpdyn_trd_zad )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_zad, ctype )   ! vertical advection 
            CASE ( jpdyn_trd_spg )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_spg, ctype )   ! surface pressure grad.
            CASE ( jpdyn_trd_dat )   ;   CALL trd_icp( ptrdx, ptrdy, jpicpd_dat, ctype )   ! damping term
            CASE ( jpdyn_trd_zdf )                                                         ! vertical diffusion 
               ! subtract surface forcing/bottom friction trends 
               ! from vertical diffusive momentum trends
               ztswu(:,:) = 0.e0   ;   ztswv(:,:) = 0.e0
               ztbfu(:,:) = 0.e0   ;   ztbfv(:,:) = 0.e0  
               DO jj = 2, jpjm1   
                  DO ji = fs_2, fs_jpim1   ! vector opt.
                     ! save the surface forcing momentum fluxes
                     ztswu(ji,jj) = taux(ji,jj) / ( fse3u(ji,jj,1)*rau0 )
                     ztswv(ji,jj) = tauy(ji,jj) / ( fse3v(ji,jj,1)*rau0 )
                     ! save bottom friction momentum fluxes
                     ikbu   = MIN( mbathy(ji+1,jj  ), mbathy(ji,jj) )
                     ikbv   = MIN( mbathy(ji  ,jj+1), mbathy(ji,jj) )
                     ikbum1 = MAX( ikbu-1, 1 )
                     ikbvm1 = MAX( ikbv-1, 1 )
                     zua = ua(ji,jj,ikbum1) * r2dt + ub(ji,jj,ikbum1)
                     zva = va(ji,jj,ikbvm1) * r2dt + vb(ji,jj,ikbvm1)
                     ztbfu(ji,jj) = - avmu(ji,jj,ikbu) * zua / ( fse3u(ji,jj,ikbum1)*fse3uw(ji,jj,ikbu) )
                     ztbfv(ji,jj) = - avmv(ji,jj,ikbv) * zva / ( fse3v(ji,jj,ikbvm1)*fse3vw(ji,jj,ikbv) )
                     !
                     ptrdx(ji,jj,1     ) = ptrdx(ji,jj,1     ) - ztswu(ji,jj)
                     ptrdy(ji,jj,1     ) = ptrdy(ji,jj,1     ) - ztswv(ji,jj)
                     ptrdx(ji,jj,ikbum1) = ptrdx(ji,jj,ikbum1) - ztbfu(ji,jj)
                     ptrdy(ji,jj,ikbvm1) = ptrdy(ji,jj,ikbvm1) - ztbfv(ji,jj)
                  END DO
               END DO
               !
               CALL trd_icp( ptrdx, ptrdy, jpicpd_zdf, ctype )   
               CALL trd_icp( ztswu, ztswv, jpicpd_swf, ctype )                               ! wind stress forcing term
               CALL trd_icp( ztbfu, ztbfv, jpicpd_bfr, ctype )                               ! bottom friction term
            END SELECT
            !
         END IF
         !
      END IF

      !>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
      ! II. Vorticity trends
      !<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

      IF( lk_trdvor .AND. ctype == 'DYN' )   THEN
         !
         SELECT CASE ( ktrd ) 
         CASE ( jpdyn_trd_hpg )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_prg )   ! Hydrostatique Pressure Gradient 
         CASE ( jpdyn_trd_keg )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_keg )   ! KE Gradient 
         CASE ( jpdyn_trd_rvo )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_rvo )   ! Relative Vorticity 
         CASE ( jpdyn_trd_pvo )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_pvo )   ! Planetary Vorticity Term 
         CASE ( jpdyn_trd_ldf )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_ldf )   ! Horizontal Diffusion 
         CASE ( jpdyn_trd_zad )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_zad )   ! Vertical Advection 
         CASE ( jpdyn_trd_spg )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_spg )   ! Surface Pressure Grad. 
         CASE ( jpdyn_trd_dat )   ;   CALL trd_vor_zint( ptrdx, ptrdy, jpvor_bev )   ! Beta V  
         CASE ( jpdyn_trd_zdf )                                                      ! Vertical Diffusion 
            ! subtract surface forcing/bottom friction trends 
            ! from vertical diffusive momentum trends
            ztswu(:,:) = 0.e0   ;   ztswv(:,:) = 0.e0
            ztbfu(:,:) = 0.e0   ;   ztbfv(:,:) = 0.e0  
            DO jj = 2, jpjm1   
               DO ji = fs_2, fs_jpim1   ! vector opt.
                  ! save the surface forcing momentum fluxes
                  ztswu(ji,jj) = taux(ji,jj) / ( fse3u(ji,jj,1)*rau0 )
                  ztswv(ji,jj) = tauy(ji,jj) / ( fse3v(ji,jj,1)*rau0 )
                  ! save bottom friction momentum fluxes
                  ikbu   = MIN( mbathy(ji+1,jj  ), mbathy(ji,jj) )
                  ikbv   = MIN( mbathy(ji  ,jj+1), mbathy(ji,jj) )
                  ikbum1 = MAX( ikbu-1, 1 )
                  ikbvm1 = MAX( ikbv-1, 1 )
                  zua = ua(ji,jj,ikbum1) * r2dt + ub(ji,jj,ikbum1)
                  zva = va(ji,jj,ikbvm1) * r2dt + vb(ji,jj,ikbvm1)
                  ztbfu(ji,jj) = - avmu(ji,jj,ikbu) * zua / ( fse3u(ji,jj,ikbum1)*fse3uw(ji,jj,ikbu) )
                  ztbfv(ji,jj) = - avmv(ji,jj,ikbv) * zva / ( fse3v(ji,jj,ikbvm1)*fse3vw(ji,jj,ikbv) )
                  !
                  ptrdx(ji,jj,1     ) = ptrdx(ji,jj,1     ) - ztswu(ji,jj)
                  ptrdx(ji,jj,ikbum1) = ptrdx(ji,jj,ikbum1) - ztbfu(ji,jj)
                  ptrdy(ji,jj,1     ) = ptrdy(ji,jj,1     ) - ztswv(ji,jj)
                  ptrdy(ji,jj,ikbvm1) = ptrdy(ji,jj,ikbvm1) - ztbfv(ji,jj)
               END DO
            END DO
            !
            CALL trd_vor_zint( ptrdx, ptrdy, jpvor_zdf )   
            CALL trd_vor_zint( ztswu, ztswv, jpvor_swf )                               ! Wind stress forcing term
            CALL trd_vor_zint( ztbfu, ztbfv, jpvor_bfr )                               ! Bottom friction term
         END SELECT
         !
      ENDIF
      !
   END SUBROUTINE trd_mod

#   else
   !!----------------------------------------------------------------------
   !!   Default case :                                         Empty module
   !!----------------------------------------------------------------------
   USE trdmod_oce      ! ocean variables trends
   USE trdvor          ! ocean vorticity trends 
   USE trdicp          ! ocean bassin integral constraints properties
   USE trdmld          ! ocean active mixed layer tracers trends 

CONTAINS

   SUBROUTINE trd_tra( kt, ktra, ktrd, ctype, ptrd2d, ptrd3d, cnbpas )
      INTEGER ::   kt, ktra, ktrd
      CHARACTER(len=3) ::   ctype   ! tracers type 'TRA' or 'TRC'
      REAL, DIMENSION(:,:)  , OPTIONAL ::   ptrd2d  ! Temperature or U trend 
      REAL, DIMENSION(:,:,:), OPTIONAL ::   ptrd3d  ! Temperature or U trend 
      CHARACTER(len=3)      , OPTIONAL ::   cnbpas  ! number of passage
      WRITE(*,*) 'trd_3d: You should not have seen this print! error ?', kt, ktra, ktrd, ctype,   &
         &                                                               ptrd2d(1,1), ptrd3d(1,1,1), cnbpas
   END SUBROUTINE trd_tra

   SUBROUTINE trd_tra_adv( kt, ktra, ktrd, ctype, pf, pun, ptn, cnbpas )
      INTEGER ::   kt, ktra, ktrd  
      CHARACTER(len=3) ::   ctype
      REAL, DIMENSION(:,:,:) ::   pf, pun, ptn
      CHARACTER(len=3), OPTIONAL ::   cnbpas
      WRITE(*,*) 'trd_3d: You should not have seen this print! error ?', kt, ktra, ktrd, ctype, pf(1,1,1),   &
         &                                                               pun(1,1,1), ptn(1,1,1), cnbpas
   END SUBROUTINE trd_tra_adv

   SUBROUTINE trd_mod(ptrd3dx, ptrd3dy, ktrd , ctype, kt, cnbpas)   ! Empty routine
      REAL, DIMENSION(:,:,:) ::   ptrd3dx, ptrd3dy    
      INTEGER ::   ktrd, kt
      CHARACTER(len=3), INTENT( in ) ::  ctype                     ! momentum or tracers trends type
      CHARACTER(len=3), INTENT( in ), OPTIONAL ::   cnbpas         ! number of passage
      WRITE(*,*) 'trd_3d: You should not have seen this print! error ?', ptrd3dx(1,1,1)
      WRITE(*,*) ' "   ": You should not have seen this print! error ?', ptrd3dy(1,1,1)
      WRITE(*,*) ' "   ": You should not have seen this print! error ?', ktrd
      WRITE(*,*) ' "   ": You should not have seen this print! error ?', ctype
      WRITE(*,*) ' "   ": You should not have seen this print! error ?', kt
      WRITE(*,*) ' "   ": You should not have seen this print! error ?', cnbpas
   END SUBROUTINE trd_mod
#   endif

   SUBROUTINE trd_mod_init
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE trd_mod_init  ***
      !! 
      !! ** Purpose :   Initialization of activated trends
      !!----------------------------------------------------------------------
      USE in_out_manager          ! I/O manager

      NAMELIST/namtrd/ ntrd, nctls, ln_trdmld_restart, ucf, ln_trdmld_instant
      !!----------------------------------------------------------------------

      IF( l_trdtra .OR. l_trddyn )   THEN
         REWIND( numnam )
         READ  ( numnam, namtrd )      ! namelist namtrd : trends diagnostic

         IF(lwp) THEN
            WRITE(numout,*)
            WRITE(numout,*) ' trd_mod_init : Momentum/Tracers trends'
            WRITE(numout,*) ' ~~~~~~~~~~~~~'
            WRITE(numout,*) '       Namelist namtrd : set trends parameters'
            WRITE(numout,*) '           * frequency of trends diagnostics   ntrd               = ', ntrd
            WRITE(numout,*) '           * control surface type              nctls              = ', nctls
            WRITE(numout,*) '           * restart for ML diagnostics        ln_trdmld_restart  = ', ln_trdmld_restart
            WRITE(numout,*) '           * instantaneous or mean ML T/S      ln_trdmld_instant  = ', ln_trdmld_instant
            WRITE(numout,*) '           * unit conversion factor            ucf                = ', ucf
        ENDIF
      ENDIF
      !
      IF( lk_trddyn .OR. lk_trdtra )    CALL trd_icp_init       ! integral constraints trends
      IF( lk_trdmld                )    CALL trd_mld_init       ! mixed-layer trends (active  tracers)  
      IF( lk_trdvor                )    CALL trd_vor_init       ! vorticity trends        
      !
   END SUBROUTINE trd_mod_init

   !!======================================================================
END MODULE trdmod
